#pragma required_pointer_size long
/*    @(#)array_type.c	1.1     2/10/94
*****************************************************************************
**                                                                          *
**                         COPYRIGHT (c) 1991 BY                            *
**             DIGITAL EQUIPMENT CORPORATION, MAYNARD, MASS.                *
**                          ALL RIGHTS RESERVED                             *
**                                                                          *
**  THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY BE USED AND  COPIED  *
**  ONLY  IN  ACCORDANCE  WITH  THE  TERMS  OF  SUCH  LICENSE AND WITH THE  *
**  INCLUSION OF THE ABOVE COPYRIGHT NOTICE.  THIS SOFTWARE OR  ANY  OTHER  *
**  COPIES  THEREOF MAY NOT BE PROVIDED OR OTHERWISE MADE AVAILABLE TO ANY  *
**  OTHER PERSON.  NO TITLE TO AND OWNERSHIP OF  THE  SOFTWARE  IS  HEREBY  *
**  TRANSFERRED.                                                            *
**                                                                          *
**  THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE  WITHOUT  NOTICE  *
**  AND  SHOULD  NOT  BE  CONSTRUED  AS  A COMMITMENT BY DIGITAL EQUIPMENT  *
**  CORPORATION.                                                            *
**                                                                          *
**  DIGITAL ASSUMES NO RESPONSIBILITY FOR THE USE OR  RELIABILITY  OF  ITS  *
**  SOFTWARE ON EQUIPMENT WHICH IS NOT SUPPLIED BY DIGITAL.                 *
**                                                                          *
*****************************************************************************
**++
**  PRODUCT:
***
**	DEC C++ Demangler
**
**  FILE:  
**
**      array_type.c
**
**  ABSTRACT:
**
**	Grammar element that parses an array type.
**
**  ENVIRONMENT:
**
**	User Mode
**
**  AUTHORS:
**
** 	Shell S. Simpson (DECwest Engineering)
**
**  MODIFICATION HISTORY:
**
**	09-Feb-2000	Shi-Jung Kao
**	Supported nontype_dim in array_type. i.e. nontype dimension.
**	Added DEMANGLE_TRACEIN/TRACEOUT.
**
**--
**/


#ifdef CXX_NT
#include "defs.h"
#include "mn_nm.h"
#include "gram_elm.h"
#include "util.h"
#else
#include "demangle.h"
#include "grammar_elements.h"
#endif

#include "demangle_internal.h"

#include <stdlib.h>
#include <stddef.h>
#include <stdio.h>
#include <string.h>

int /* recognition status */
_array_type(const char* s,		/* IN:  string to be parsed */
           int start,			/* IN:  index of first unparsed character */
           int* end,			/* OUT: index of last parsed character */
           struct MLD_array** p)	/* OUT: structure containing array (allocated) */
{
  /* variables common to all language elements */
  int ci;  /* current index */
  char ch; /* current character */
  
  /* variables particular to this language element */
  int status;
  int end_of_type;
  struct MLD_type* t; /* storage for a type */
  struct MLD_array* temp_p;
  int d;			/* Array integer dim. */
  struct MLD_type* dt = 0L;	/* Array nontype_dim. */

  DEMANGLE_TRACEIN(_array_type);
    
  /* initialization particular to language element */

  *p = 0L;
  *end = start - 1;

  /*
  **
  ** Check for Array Token
  **
  */

  ch = s[ci = start]; /* initialize ch and ci */
  if (ch != ARRAY_TOKEN)
  {
    DEMANGLE_TRACEOUT(_array_type);
    return MLD_NOTHING_RECOGNIZED;
  }
  ch = s[++ci]; /* increment ch and ci */

  /*
  **
  ** Check for dimension of array
  **
  */

  status = _number(s, ci, &end_of_type, &d); /* d contains the dimension */
  
  switch(status)
  {
    case MLD_NOTHING_RECOGNIZED:
      if (strncmp(&s[ci], "_Z", 2) == 0) {
	ci++;
	status = _nontype_type(s, FALSE/*is_pointer*/, ci, &end_of_type, &dt);
	if (status == MLD_SOMETHING_RECOGNIZED)
	  break;
      }
      DEMANGLE_TRACEOUT(_array_type);
      return MLD_ERROR;

    case MLD_ERROR:
      DEMANGLE_TRACEOUT(_array_type);
      return MLD_ERROR;
      break;
    
    case MLD_SOMETHING_RECOGNIZED:
      /* normal case, continue */
      break;
  }

  /*
  **
  ** Check for '_'
  **
  */

  ch = s[ci = (end_of_type+1)];
  if (ch != '_') /* if there isn't an underscore, error */
  {
    _delete_type_structure(dt);
    DEMANGLE_TRACEOUT(_array_type);
    return MLD_NOTHING_RECOGNIZED;
  }
  ch = s[++ci]; /* increment ch and ci */

  /*
  **
  ** Check for type
  **
  */

  status = _type(s, ci, &end_of_type, &t);
  
  switch(status)
  {
    case MLD_NOTHING_RECOGNIZED: /* Must be a type after the Array token */
    case MLD_ERROR:
      _delete_type_structure(dt);
      DEMANGLE_TRACEOUT(_array_type);
      return MLD_ERROR;
    break;
    
    case MLD_SOMETHING_RECOGNIZED:
      /* normal case, continue */
    break;
  }

  /*
  **
  ** Put the Array structure together
  **
  */

  temp_p = (struct MLD_array*) malloc(sizeof(struct MLD_array)); 
  DEMANGLE_RECORD_ALLOC(temp_p);
  if (temp_p == 0L) return MLD_ERROR; /* no memory allocated, error */
  
  temp_p->dimension = d;
  temp_p->type_modified = t;
  temp_p->nontype_dim = dt;
#if !PRODUCTION
  if (demangle_dump)
    _dump_array(temp_p);
#endif
  *p = temp_p;
  *end = end_of_type; 
  DEMANGLE_TRACEOUT(_array_type);
  return MLD_SOMETHING_RECOGNIZED;
}
