#pragma required_pointer_size long
#ifndef lint
static const char sccsid[] = "@(#)complex_type.c	1.1     2/10/94";
#endif
/*
*****************************************************************************
**                                                                          *
**                         COPYRIGHT (c) 1991 BY                            *
**             DIGITAL EQUIPMENT CORPORATION, MAYNARD, MASS.                *
**                          ALL RIGHTS RESERVED                             *
**                                                                          *
**  THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY BE USED AND  COPIED  *
**  ONLY  IN  ACCORDANCE  WITH  THE  TERMS  OF  SUCH  LICENSE AND WITH THE  *
**  INCLUSION OF THE ABOVE COPYRIGHT NOTICE.  THIS SOFTWARE OR  ANY  OTHER  *
**  COPIES  THEREOF MAY NOT BE PROVIDED OR OTHERWISE MADE AVAILABLE TO ANY  *
**  OTHER PERSON.  NO TITLE TO AND OWNERSHIP OF  THE  SOFTWARE  IS  HEREBY  *
**  TRANSFERRED.                                                            *
**                                                                          *
**  THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE  WITHOUT  NOTICE  *
**  AND  SHOULD  NOT  BE  CONSTRUED  AS  A COMMITMENT BY DIGITAL EQUIPMENT  *
**  CORPORATION.                                                            *
**                                                                          *
**  DIGITAL ASSUMES NO RESPONSIBILITY FOR THE USE OR  RELIABILITY  OF  ITS  *
**  SOFTWARE ON EQUIPMENT WHICH IS NOT SUPPLIED BY DIGITAL.                 *
**                                                                          *
*****************************************************************************
**++
**  PRODUCT:
***
**	DEC C++ Demangler
**
**  FILE:  
**
**      complex_type.c
**
**  ABSTRACT:
**
**	Grammar element that parses a complex type.
**
**  ENVIRONMENT:
**
**	User Mode
**
**  AUTHORS:
**
** 	Shell S. Simpson (DECwest Engineering)
**
**  MODIFICATION HISTORY:
**
**  30-Dec-1997	 Shi-Jung Kao
**	Processed UNNAMED tokens.
**
**  3-Nov-1999	 Shi-Jung Kao
**	Supported nontype in complex_type. i.e. ZnZ.
**
**  24-Jan-2000  Shi-Jung Kao
**	Initial *c to 0L.
**
**  28-Jan-2000  Shi-Jung Kao
**	Accepted foo__tm__2_i without function args.
**	Increast "string" size from STRSIZ to BUFSIZ.
**
**  07-Feb-2000  Shi-Jung Kao
**	Accepted unnamed namespace with a module name.
**	e.g. __Nt_cxx_main where module t.cxx(main)
**      Added DEMANGLE_TRACEIN/TRACEOUT.
**      Moved code to utilities.c, and called it demangle_unnamed.
**      Skip foo__tm_2_i here, let mangled_template_name handle it.
**
**--
**/


#ifdef CXX_NT
#include "defs.h"
#include "mn_nm.h"
#include "gram_elm.h"
#include "util.h"
#else
#include "demangle.h"
#include "grammar_elements.h"
#endif

#include "demangle_internal.h"

#include <stdlib.h>
#include <stddef.h>
#include <stdio.h>
#include <string.h>

/* language element particular definitions */
#define SPACE 32
#define TAB 9
#define CARRIAGE_RETURN 13
#define LINE_FEED 10


int /* recognition status */
_complex_type(const char* s,		/* IN:  string to be parsed */
            int start,			/* IN:  index of first unparsed character */
            int* end,			/* OUT: index of last parsed character */
            struct MLD_complex** c)	/* OUT: complex type */
{
  int status;
  char string[BUFSIZ];
  char bs[STRSIZ];
  char rs[STRSIZ];
  char as[STRSIZ];
  int end_of_pascal_string;
  int end_of_gc;
  struct MLD_complex* temp_c;
  struct MLD_mangled_template_name* temp_mtn;
  int ci = start;
  int rtn_end;
  int temp_id = 0;
  char ch;
  int template_flag = 0;

  *c = 0L;
  *end = start - 1;
  if (start > strlen(s))
    return MLD_NOTHING_RECOGNIZED;
  DEMANGLE_TRACEIN(_complex_type);

#if DEC_ANSI_FORM
  if (ci > 1 && strncmp(&s[ci-2], "__tm__", 6) == 0) {
    ci = ci + 4;
    template_flag = TRUE;
  }

  /* check if is nontype ZnZ style. */
  if (s[ci] == 'Z') {
    int end_of_ZnZ;
    int cj = ci;
    struct MLD_type* t = 0L;
    status = _nontype_type(s, FALSE/*is_pointer*/, cj, &end_of_ZnZ, &t);
    if (status != MLD_SOMETHING_RECOGNIZED)
      {
	*end = start - 1;
	*c = 0L;
	DEMANGLE_TRACEOUT(_complex_type);
	return status;
      }
    *end = end_of_ZnZ;
    temp_mtn = (struct MLD_mangled_template_name*) 
		malloc(sizeof(struct MLD_mangled_template_name));
    if (temp_mtn == 0L) {
      DEMANGLE_TRACEOUT(_complex_type);
      return MLD_ERROR; /* error if no memory allocated */
    }
    DEMANGLE_RECORD_ALLOC(temp_mtn);

    temp_mtn->identifier = 0L;
    temp_mtn->args = t;

    temp_c = (struct MLD_complex*) malloc(sizeof(struct MLD_complex)); 
    if (temp_c == 0L) {
      DEMANGLE_TRACEOUT(_complex_type);
      return MLD_ERROR;
    }
    DEMANGLE_RECORD_ALLOC(temp_c);

    temp_c->name.template_name = temp_mtn;
    temp_c->mangled_template_name = 1; /* TRUE */
    *c = temp_c;
    DEMANGLE_TRACEOUT(_complex_type);
    return MLD_SOMETHING_RECOGNIZED;
  }
#endif
  /* 
  **
  ** Step 1:  Get a pascal string
  **
  */

  status = _pascal_string(s, ci, &end_of_pascal_string, string, BUFSIZ);
  if (status != MLD_SOMETHING_RECOGNIZED)
  {
    *end = start - 1;
    *c = 0L;
    DEMANGLE_TRACEOUT(_complex_type);
    return status;
  }
  
  /* 
  **
  ** Step 2:  Check to see if this pascal string is a mangled
  **          template name
  **
  */
  
  if (template_flag) {
    /* Reconstruct the string. e.g. too__tm__3_df in a
    ** case of mangled name:
    **	too__tm__3_df__XRC17Vector__tm__4_Z2Z_17Vector__tm__4_Z1Z
    **  ci = 10, end_of_pascal_string = 12, add 2 to skip the "__".
    **
    ** Support the case of: foo__tm__2_i and without function args.
    ** i.e. there is no "__" after it but the '\0' or '\n'.
    */
    strncpy(string, s, end_of_pascal_string+1);
    string[end_of_pascal_string+1] = '\0';
    if (s[end_of_pascal_string+1] != '\0' && s[end_of_pascal_string+1] != '\n')
      end_of_pascal_string = end_of_pascal_string + 2;
    else {
      /* Skip this, let mangled_tempalte_name handle it. */
      *end = start - 1;
      *c = 0L;
      DEMANGLE_TRACEOUT(_complex_type);
      return MLD_NOTHING_RECOGNIZED;
    }
  }

  while (string[temp_id] == ' ')
    temp_id++;
  if (MLD_mangled_version == MLD_VERSION7 && 
      strncmp(&string[temp_id], VERSION7, VERSION_STRING_LENGTH) == 0)
    status = _mangled_template_name(string, temp_id+VERSION_STRING_LENGTH, 
				    &end_of_gc, &temp_mtn,
				    bs, rs, as, STRSIZ);
  else
    status = _mangled_template_name(string, 0, &end_of_gc, &temp_mtn,
				    bs, rs, as, STRSIZ);
  switch (status)
  {
    case MLD_ERROR:  /* something bad has happened */
      *end = start - 1;
      *c = 0L;
      DEMANGLE_TRACEOUT(_complex_type);
      return MLD_ERROR;  
      break;
    
    case MLD_NOTHING_RECOGNIZED: /* we just have a normal structure or class */
      *end = end_of_pascal_string;
      
      temp_c = (struct MLD_complex*) malloc(sizeof(struct MLD_complex)); 
      DEMANGLE_RECORD_ALLOC(temp_c);
      if (temp_c == 0L) {
	DEMANGLE_TRACEOUT(_complex_type);
	return MLD_ERROR;
      }

      demangle_unnamed (string, string);

      temp_c->name.structure_name = malloc(strlen(string)+1);
      DEMANGLE_RECORD_ALLOC(temp_c->name.structure_name);
      if (temp_c->name.structure_name == 0L) 
      {
	DEMANGLE_FREE(temp_c);
	DEMANGLE_TRACEOUT(_complex_type);
        return MLD_ERROR;
      }
      
      strcpy(temp_c->name.structure_name, string);
      
      temp_c->mangled_template_name = 0; /* FALSE */
      
      *c = temp_c;
      DEMANGLE_TRACEOUT(_complex_type);
      return MLD_SOMETHING_RECOGNIZED;
      break;
    
    case MLD_SOMETHING_RECOGNIZED: /* we have a template name...maybe */
      *end = end_of_pascal_string;
      
      /* the whole string must be a complex */
      if ((end_of_gc + 1) != strlen(string)
#if DEC_ANSI_FORM
	  && end_of_gc != strlen(string)
#endif
	  ) {
	DEMANGLE_TRACEOUT(_complex_type);
	return MLD_NOTHING_RECOGNIZED;
      }

      temp_c = (struct MLD_complex*) malloc(sizeof(struct MLD_complex)); 
      DEMANGLE_RECORD_ALLOC(temp_c);
      if (temp_c == 0L) {
	DEMANGLE_TRACEOUT(_complex_type);
	return MLD_ERROR;
      }
      
#if DEC_ANSI_FORM
//    Don't use following code, keep the identifier.  We may change 
//    code later.
//    if (template_flag && temp_mtn->identifier != 0L) {
//      DEMANGLE_FREE(temp_mtn->identifier);
//      temp_mtn->identifier = (char*) malloc(8+1);
//      strcpy(temp_mtn->identifier, "Template");
//      DEMANGLE_RECORD_ALLOC(temp_mtn->identifier);
//    }
#endif

      temp_c->name.template_name = temp_mtn;
      
      temp_c->mangled_template_name = 1; /* TRUE */

      *c = temp_c;

      DEMANGLE_TRACEOUT(_complex_type);
      return MLD_SOMETHING_RECOGNIZED;
      break;  
  }
  DEMANGLE_TRACEOUT(_complex_type);
}
