#pragma required_pointer_size long
#ifndef lint
static const char sccsid[] = "@(#)constant_type.c	1.1      2/10/94";
#endif
/*
*****************************************************************************
**                                                                          *
**                         COPYRIGHT (c) 1991 BY                            *
**             DIGITAL EQUIPMENT CORPORATION, MAYNARD, MASS.                *
**                          ALL RIGHTS RESERVED                             *
**                                                                          *
**  THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY BE USED AND  COPIED  *
**  ONLY  IN  ACCORDANCE  WITH  THE  TERMS  OF  SUCH  LICENSE AND WITH THE  *
**  INCLUSION OF THE ABOVE COPYRIGHT NOTICE.  THIS SOFTWARE OR  ANY  OTHER  *
**  COPIES  THEREOF MAY NOT BE PROVIDED OR OTHERWISE MADE AVAILABLE TO ANY  *
**  OTHER PERSON.  NO TITLE TO AND OWNERSHIP OF  THE  SOFTWARE  IS  HEREBY  *
**  TRANSFERRED.                                                            *
**                                                                          *
**  THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE  WITHOUT  NOTICE  *
**  AND  SHOULD  NOT  BE  CONSTRUED  AS  A COMMITMENT BY DIGITAL EQUIPMENT  *
**  CORPORATION.                                                            *
**                                                                          *
**  DIGITAL ASSUMES NO RESPONSIBILITY FOR THE USE OR  RELIABILITY  OF  ITS  *
**  SOFTWARE ON EQUIPMENT WHICH IS NOT SUPPLIED BY DIGITAL.                 *
**                                                                          *
*****************************************************************************
**++
**  PRODUCT:
***
**	DEC C++ Demangler
**
**  FILE:  
**
**      constant_type.c
**
**  ABSTRACT:
**
**	Grammar element that parses a constant type.
**
**  ENVIRONMENT:
**
**	User Mode
**
**  AUTHORS:
**
** 	Shell S. Simpson (DECwest Engineering)
**
**  MODIFICATION HISTORY:
**
**  18-Mar-1998	Shi-Jung Kao
**      DEC C++ V60 support. pointer to member (function) constant.
**
**  22-Apr-1999 Shi-Jung Kao
**	Compaq C++ V6.2 changes fromat from L212 into L_2_12.
**
**  30-Nov-1999 Shi-Jung Kao
**	Support ANSI negative char of "n".
**
**  20-Jan-2000 Shi-Jung Kao
**	Accept constant type of pointer to member "LM0..."
**	Added DEMANGLE_TRACEIN/TRACEOUT.
**
**--
**/



#ifdef CXX_NT
#include "defs.h"
#include "mn_nm.h"
#include "gram_elm.h"
#include "util.h"
#else
#include "demangle.h"
#include "grammar_elements.h"
#endif

#include "demangle_internal.h"

#include <stdlib.h>
#include <stddef.h>
#include <stdio.h>
#include <string.h>

/* language element particular definitions */
#define ASCII_ZERO 48

int /* recognition status */
_constant_type(const char* s,	/* IN:  string to be parsed */
      		  int start,	/* IN:  index of first unparsed character */
       		  int* end,		/* OUT: index of last parsed character */
       		  char* c,		/* OUT: string to contain constant */
       		  int max_c)	/* IN:	maximum size of c */
{
  int start_of_constant;
  int end_of_constant;
  int size_of_constant;
  int temp;
  int end_of_grammar_element;
  int status = MLD_NOTHING_RECOGNIZED;
  int minus_id = 0;

  /* variables common to all language elements */
  int ci;  /* current index */
  char ch; /* current character */
  ch = s[ci = start]; /* initialize ch and ci */
  
  DEMANGLE_TRACEIN(_constant_type);
  if (ch == 'L' && strlen(s) > start+1 && s[start+1] == 'M') {
    /* Ok, a constant type of pointer to member "LM0..." */
  } else {
    if (ch != '_')
      {
	DEMANGLE_TRACEOUT(_constant_type);
	return MLD_NOTHING_RECOGNIZED;
      }
    ch = s[++ci]; /* increment ch and ci */
  }
  start_of_constant = ci;
  
  /* get "-" if present */
  if (ch == '-') {
    ch = s[++ci]; /* increment ch and ci */
  } else if (ch == 'n') { /* ANSI negative char */
    minus_id = ci;
    ch = s[++ci]; /* increment ch and ci */
  }

  if (ch == 'L') {
      
    /* Pointer to member:
	 For pointers to data members, the offset value encoded as an integer:
	 L212  <--- encoding for an offset of "12"
           ^^------ Literal value.
	  ^-------- Length of the literal.
	 ^--------- "L" indicates a number.
	 For pointers to member functions, the __mptr triplet of
	 values (delta, index, function or offset), encoded as follows:
	 LM0_L2n1_1j
                  ^^- Function name, or alternatively "0" if the pointer
                      to member uses an offset (e.g., LM0_L11_0).
             ^^^^---- Index value, encoded as an integer.
           ^--------- Delta value.
	 ^^---------- "LM" indicates a pointer to member function.
     This is compatible with cfront 3.0.1.  Note that "0" is always
     used for the offset, not the actual offset value.  This follows
     cfront.  The idea seems to be that "0" is really a way of saying
     "there is no function;" the offset value itself would not be
     of interest to a name demangler. */

    /* DEC C++ V60 emits L212 (ptr to member), output L212 to separate from V62.
     * Changes: Compaq C++ V6.2 emits L_2_12_  instead.
     * Check "L_" for V6.2 case, and return string "12\0".
     */

    if (s[ci+1] == '_') {
      status = _constant_type2(s, 3/* int basic_type in definitions.h */,
			       ci, &end_of_grammar_element, c, max_c);
      /* It has to be the form of L_n_nn_ */
      if (status == MLD_SOMETHING_RECOGNIZED && 
			s[end_of_grammar_element+1] == '_') {
	*end = end_of_grammar_element + 1;
	DEMANGLE_TRACEOUT(_constant_type);
	return status;
      } else {
	DEMANGLE_TRACEOUT(_constant_type);
	return MLD_NOTHING_RECOGNIZED;
      }
    }

    ch = s[++ci];
    if (ch == 'M') {
      int status;
      char string[STRSIZ];
      char string2[STRSIZ];
      int end_of_pascal_string;
      int len;

      /* DEC C++ V60: e.g. LM0_L_2_n1_1j (ptr to member function "j") */
      ch = s[++ci];

      while ((ch >= '0') && (ch <= '9')) /* get digits */  
	ch = s[++ci]; /* increment ch and ci */
      if (ci+7 > strlen(s) ||
	  ch != '_' || s[ci+1] != 'L' || s[ci+2] != '_') {
	DEMANGLE_TRACEOUT(_constant_type);
	return MLD_NOTHING_RECOGNIZED;
      }
      ci = ci + 3;
      ch = s[ci];
      while ((ch >= '0') && (ch <= '9')) /* get digits */  
	ch = s[++ci]; /* increment ch and ci */
      if (ch != '_') {
	DEMANGLE_TRACEOUT(_constant_type);
	return MLD_NOTHING_RECOGNIZED;
      }
      ch = s[++ci];
      if (ch == 'n')
	ch = s[++ci];
      while ((ch >= '0') && (ch <= '9')) /* get digits */  
	ch = s[++ci]; /* increment ch and ci */
      if (ch != '_') {
	DEMANGLE_TRACEOUT(_constant_type);
	return MLD_NOTHING_RECOGNIZED;
      }
      status = _pascal_string(s, ci+1, &end_of_pascal_string, string, STRSIZ);
      if (status != MLD_SOMETHING_RECOGNIZED) {
	DEMANGLE_TRACEOUT(_constant_type);
	return MLD_NOTHING_RECOGNIZED;
      }
      {
	int num_du = 0;		/* number of double underscores */
	int du_count;	        /* looping variable that determines the
                                   underscore number that will be used
                                   in the identifier */
	char id[STRSIZ];	/* identifier string */
	char bs[STRSIZ];	/* string before demangled string */
	char string2[STRSIZ];	/* string before demangled string */
	int end_of_component;
	struct MLD_qualification* q=0;/* qualifier */

	num_du = _num_dbl_uscores(string, 0);

	for (du_count=1; du_count<=num_du; du_count++) { 
	  /* start of loop that repeatled check for double underscores */
	  ch = string[ci = 0]; /* initialize ci and ch */
	  status = _identifier(string, ci, &end_of_component, du_count,
                                           id, STRSIZ, bs, STRSIZ);
	  if (status == MLD_NOTHING_RECOGNIZED || status == MLD_ERROR)
	    break;
	  ch = string[ci = (end_of_component+1)]; /* update ci and ch */
	  /*
	  **
	  ** Get the "__"
	  **
	  */
	  if (!((string[ci] == '_') && (string[ci+1] == '_')))
	    continue;

	  ch = string[ci = (ci+2)]; /* update ci and ch */
	  /*
	  **
	  ** Get the qualifications
	  **
	  */
	  status = _parse_qualifications(string, ci, &end_of_component, &q);
	  if (status == MLD_NOTHING_RECOGNIZED || status == MLD_ERROR)
	    break;
	  if (end_of_component+1 == strlen(string)) {
	    string2[0] = 0;
	    _concat_qualifications(q, string2, STRSIZ, 0/*flags*/);
	    _concat(string2, STRSIZ, "::");
	    _concat(string2, STRSIZ, id);
	    strcpy(string, string2);
	    _delete_qualifications(q);
	  }
	}
      }
      len = strlen(string);
      strncpy(c, string, len);
      c[len] = ' ';
      c[len+1] = '*';
      c[len+2] = 0;
      *end = end_of_pascal_string + 1;
      DEMANGLE_TRACEOUT(_constant_type);
      return MLD_SOMETHING_RECOGNIZED;
    }
  }
  temp = ci;
  /* get digits */  
  while ((ch >= '0') && (ch <= '9'))
  {
      ch = s[++ci]; /* increment ch and ci */
  }
  end_of_constant = ci - 1;
  /* return nothing recognized if no characters recognized */
  if (temp == ci) {
    DEMANGLE_TRACEOUT(_constant_type);
    return MLD_NOTHING_RECOGNIZED;
  }
  
  /* check for an ending '_' */
  if (ch != '_')
  {
    DEMANGLE_TRACEOUT(_constant_type);
    return MLD_NOTHING_RECOGNIZED;
  }
  
  /* At this point we know we have a constant, time to copy */
  size_of_constant = end_of_constant - start_of_constant + 1;
  if (size_of_constant < max_c)
  {
    strncpy(c, &s[start_of_constant], size_of_constant);
    c[size_of_constant] = 0;
    if (minus_id == start_of_constant)
      c[0] = '-'; /* Under ANSI replace "n" with "-". */
  }
  else
  {
    DEMANGLE_TRACEOUT(_constant_type);
    return MLD_ERROR;  /* NO SPACE !!!! */
  }
  
  
  *end = ci; /* ci is the last digit parsed */
  DEMANGLE_TRACEOUT(_constant_type);
  return MLD_SOMETHING_RECOGNIZED;
}

int /* recognition status */
_constant_type2(const char* s,	/* IN:  string to be parsed */
	       int basic_type,  /* IN:  index of basic type */
	       int start,	/* IN:  index of first unparsed character */
	       int* end,	/* OUT: index of last parsed character */
	       char* c,		/* OUT: string to contain constant */
	       int max_c)	/* IN:	maximum size of c */
{
  /* Looking for: L_2_98_
   * return string of "98\0" or "b\0" if basic_type is char,
   * and point *end at the ending "_".
   */


  int start_of_constant;
  int end_of_constant;
  int size_of_constant;
  int temp;
  int length_of_s = strlen(s);
  int status;
  int end_of_component;
  int num_of_digits = 0;
  char constant_str[32];
  int i;

  /* variables common to all language elements */
  int ci;  /* current index */
  char ch; /* current character */
  ch = s[ci = start]; /* initialize ch and ci */

  if (length_of_s < ci+1 || ch != 'L' || s[ci+1] != '_') {
    return MLD_NOTHING_RECOGNIZED;
  }

  status = _number(s, ci+2, &end_of_component, &num_of_digits);
  if (status != MLD_SOMETHING_RECOGNIZED || num_of_digits == 0 ||
      length_of_s <end_of_component+1 || s[end_of_component+1] != '_' ||
      num_of_digits >= max_c)
    return MLD_NOTHING_RECOGNIZED;
  if (basic_type == 1) {
    /* char type */
    for (i=0; i<num_of_digits; i++) {
      constant_str[i] = s[end_of_component+2+i];
    }
    constant_str[i] = '\0';
    c[0] = atoi(&constant_str[0]);
    c[1] = '\0';
  } else {
    for (i=0; i<num_of_digits; i++) {
      c[i] = s[end_of_component+2+i];
    }
  }
  c[i] = '\0';
  
  *end = end_of_component+1+num_of_digits; /* ci is the last digit parsed */
  return MLD_SOMETHING_RECOGNIZED;
}

int /* TRUE or FALSE */
_is_special_constant_basic_type(struct MLD_type* t, int* id)
{
  /* Note, this is not a general purpose routine. */
  if (t->tm_list) {
    if (t->tm_list->const_tm &&
	t->ut && t->ut->type_of_type == MLD_BASIC_TYPE) {
      *id = t->ut->the_type.basic;
      return 1;
    } else {
      return 0;
    }
  } else {
    struct MLD_type_modifiers* tl = 0L;
    struct MLD_type*	type_modified = 0L;

    if (!(t->ut))
      return 0;
    switch (t->ut->type_of_type) {
    case MLD_POINTER_TO_MEMBER_TYPE:
      type_modified = t->ut->the_type.ptr_to_member->type_modified;
      break;

    case MLD_POINTER_TYPE:
      type_modified = t->ut->the_type.pointer->type_modified;
      break;

    case MLD_REFERENCE_TYPE:
      type_modified = t->ut->the_type.reference->type_modified;
      break;

    case MLD_ARRAY_TYPE:
      type_modified = t->ut->the_type.array->type_modified;
      break;

    case MLD_CONSTANT_TYPE:
    case MLD_FUNCTION_TYPE:
    case MLD_COMPLEX_TYPE:
    default:
      return 0;
    }
    tl = type_modified ? type_modified->tm_list : 0L;
    if (tl && tl->const_tm &&
	type_modified->ut->type_of_type == MLD_BASIC_TYPE) {
      *id = type_modified->ut->the_type.basic;
      return 1;
    } else
      return 0;
  }
}

int /* recognition status */
_make_constant_type(char* c,		/* IN: Constant string */
		    struct MLD_type** tt) /* OUT: MLD_type */
{
  struct MLD_type* t; /* type */
  struct MLD_unmodified_type* ut; /* unmodified type */

  DEMANGLE_TRACEIN(_make_constant_type);  
  /* create unmodified type */
    ut = (struct MLD_unmodified_type*) 
			malloc(sizeof(struct MLD_unmodified_type));
    if (ut == 0L) {
      DEMANGLE_TRACEOUT(_make_constant_type);  
      return MLD_ERROR;  /* error, no memory allocated */
    }
    DEMANGLE_RECORD_ALLOC(ut);

    ut->type_of_type = MLD_CONSTANT_TYPE;
    ut->the_type.type_name = (char*) malloc(strlen(c)+1);
    ut->shared_id = 0L;
    if (ut->the_type.type_name == 0L) {
      DEMANGLE_FREE(ut);
      DEMANGLE_TRACEOUT(_make_constant_type);  
      return MLD_ERROR;
    }
    DEMANGLE_RECORD_ALLOC(ut->the_type.type_name);

    strcpy(ut->the_type.type_name, c);
  
    /* create type structure */
    t = (struct MLD_type*) malloc(sizeof(struct MLD_type)); 
    if (t == 0L) {
      DEMANGLE_FREE(ut->the_type.type_name);
      DEMANGLE_FREE(ut);
      DEMANGLE_TRACEOUT(_make_constant_type);  
      return MLD_ERROR; /* error, no memory allocated */
    }
    DEMANGLE_RECORD_ALLOC(t);
    
    t->ut = ut;
    t->shared_ut = 0;
    t->tm_list = 0L; /* no type modifiers */
    t->next = 0L; /* no next, yet */
    t->repeat_type = 0L;
    t->repeat_count = 0;
      
    *tt = t;
    DEMANGLE_TRACEOUT(_make_constant_type);  
    return MLD_SOMETHING_RECOGNIZED;
}
