/*
 * @DEC_COPYRIGHT@
 */
/*
 * HISTORY
 * $Log: demangle_shared.c,v $
 * Revision 1.1.4.2  1999/07/15  17:08:54  John_P_Williams
 * 	Enablers for libdemangle.
 * 	[1999/07/13  21:26:42  John_P_Williams]
 *
 * $EndLog$
 */
#pragma ident "@(#)$RCSfile: demangle_shared.c,v $ $Revision: 1.1.4.2 $ (DEC) $Date: 1999/07/15 17:08:54 $"

/*
** C++ DEMANGLING CODE OVERRIDE
**
** This is libmld's new entry point for the C++ demangler. It tries to
** dynamically load libdemangle.so (that a C++ kit may have installed),
** and to use the demangler in that library if that version is newer
** than the one in libmld.a. The version is indicated by the constant
** __MLD_demangle_version. The library's demangler is executed via
** its __MLD_demangle_string entry point.
**
** This module is delivered only as part of libmld.a, not in any 
** libdemangle.{a,so} library, which do not define MLD_demangle_string.
**
** See demangle_string.c for a complete description of the design and
** the implementation of __MLD_demangle_version and __MLD_demangle_string.
*/

#include <stdio.h>
#include <dlfcn.h>

extern unsigned long __MLD_demangle_version;
extern int __MLD_demangle_string();

static int (*demangle_string_ptr)() = __MLD_demangle_string;

static int tried_dlopen = 0;

static load_dynamic_routines()
{
  void *handle = dlopen(
#if defined(DEC_EXX_LINUX)
			"libcompaq_demangle.so",
#else
			"libdemangle.so",
#endif
			RTLD_LAZY);

  if (handle != NULL) {
    unsigned long *version = (unsigned long *)
      dlsym(handle,"__MLD_demangle_version");

    if (version != NULL && *version > __MLD_demangle_version) {

      demangle_string_ptr = (int (*)())dlsym(handle,"__MLD_demangle_string");
      if (demangle_string_ptr == NULL) {
	demangle_string_ptr = __MLD_demangle_string;
      }
    }

    dlclose(handle);
  }
  tried_dlopen = 1;
}

int /* status */
MLD_demangle_string(const char* s, /* IN:  string to be demangled */
                    char* ds,      /* OUT: demangled string */
                    int ds_max,    /* IN:  maximum size of ds */
                    int flags)     /* IN:  flags to control string formation */
{
    if (!tried_dlopen) {
      load_dynamic_routines();
    }

    return (*demangle_string_ptr)(s,ds,ds_max,flags);
}

