#ifndef lint
/* static const char sccsid[] = "@(#)demangle_string.h	1.1     2/10/94"; */
#endif
/*
*****************************************************************************
**                                                                          *
**                         COPYRIGHT (c) 1991 BY                            *
**             DIGITAL EQUIPMENT CORPORATION, MAYNARD, MASS.                *
**                        ALL RIGHTS RESERVED                               *
**                                                                          *
**  THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY BE USED AND  COPIED  *
**  ONLY  IN  ACCORDANCE  WITH  THE  TERMS  OF  SUCH  LICENSE AND WITH THE  *
**  INCLUSION OF THE ABOVE COPYRIGHT NOTICE.  THIS SOFTWARE OR  ANY  OTHER  *
**  COPIES  THEREOF MAY NOT BE PROVIDED OR OTHERWISE MADE AVAILABLE TO ANY  *
**  OTHER PERSON.  NO TITLE TO AND OWNERSHIP OF  THE  SOFTWARE  IS  HEREBY  *
**  TRANSFERRED.                                                            *
**                                                                          *
**  THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE  WITHOUT  NOTICE  *
**  AND  SHOULD  NOT  BE  CONSTRUED  AS  A COMMITMENT BY DIGITAL EQUIPMENT  *
**  CORPORATION.                                                            *
**                                                                          *
**  DIGITAL ASSUMES NO RESPONSIBILITY FOR THE USE OR  RELIABILITY  OF  ITS  *
**  SOFTWARE ON EQUIPMENT WHICH IS NOT SUPPLIED BY DIGITAL.                 *
**                                                                          *
*****************************************************************************
**++
**  PRODUCT:
***
**	DEC C++ Demangler
**
**  FILE:  
**
**      demangle_string.h
**
**  ABSTRACT:
**
**	Definitions necessary for demangling a string.
**
**  ENVIRONMENT:
**
**	User Mode
**
**  AUTHORS:
**
** 	Shell S. Simpson (DECwest Engineering)
**
**  MODIFICATION HISTORY:
**
**  Feb-16-2000  Shi-Jung Kao
**	Updated the function interface, flags, status.
**
**--
**/


#ifndef _DEMANGLE_STRING_H_
#define _DEMANGLE_STRING_H_

/*
**
**  DEMANGLER FLAGS
**
**  The following flags can be passed to the "MLD_demangle_string" 
**  function as the 4th argument "flags".  To combine several flags, 
**  logically OR them together.
**  NOTE:  combining MLD_SHOW_MANGLED_NAME and MLD_SHOW_DEMANGLED_NAME 
**	   will put the mangled name in parens.
** 
*/

#define MLD_SHOW_MANGLED_NAME	0x01	/* mangled name */
#define MLD_SHOW_INFO		0x02	/* reserved, no data */
#define MLD_NO_SPACES		0x04	/* no space in output */
#define MLD_SHOW_DEMANGLED_NAME	0x08	/* demangled name */
#define MLD_DUMP		0x10	/* reserved for development */
#define MLD_MEMCHECK		0x20	/* reserved for development */
#define MLD_NO_TEMPLATE_DETAILS	0x40	/* reserved for development */

/*
**  The routine MLD_demangle_string is in the libmld.a (or libdemangle.a)
*/

/*
**
**  DEMANGLE STRING
**
**  The "MLD_demangle_string" function, demangles mangled names into user
**  friendly names, and return an int of status.
**
**  There are 4 arguments.
**	char* s:	string (characters terminated with a null character)
**			to be demangled, may contain multiple mangled names
**			separated with a "space".
**	char* ds:	tell the MLD_demangle_string where the result string 
**			of demangled names should be stored at.
**	int   ds_max:	tell the MLD_demangle_string about the maximum 
**			available size of ds is.
**	int   flags:	tell MLD_demangle_string what to do.  Details 
**			see above.
**
**  If there are several mangled names, all of the mangled names are replaced
**  with demangled names.  If there are no mangled names or an error occurs,
**  "ds" will be equal to "s".  If ds is not large enough to hold the demangled
**  name, an error will result (and consequently, as just mentioned, "ds" will
**  equal "s").
**
**  NOTE:  This function will return an int, represents the status of 
**	demangling, whose value is one of the following:
**
**	MLD_SOMETHING_RECOGNIZED
**	MLD_NOTHING_RECOGNIZED
**	MLD_ERROR
**	MLD_INADEQUATE_ARRAY_SIZE
*/

/*
**
**  DEMANGLER STATUS
**
*/
#define MLD_SOMETHING_RECOGNIZED 	 1
#define MLD_NOTHING_RECOGNIZED 		 0
#define MLD_ERROR			-1
#define MLD_INADEQUATE_ARRAY_SIZE 	-2

#ifdef __cplusplus
extern "C" {
#endif
int /* status */ 
MLD_demangle_string (const char* s,	/* IN:  string to be demangled */
                     char* ds,		/* OUT: demangled string */
                     int ds_max,	/* IN:	maximum size of ds */
                     int flags); /* IN:	flags to control string formation */
#ifdef __cplusplus
}
#endif

#endif /* _DEMANGLE_STRING_H_ */
/* DON'T ADD ANYTHING AFTER THIS #endif */
