#pragma required_pointer_size long
#ifndef lint
static const char sccsid[] = "@(#)function_type.c	1.1     2/10/94";
#endif
/*
*****************************************************************************
**                                                                          *
**                         COPYRIGHT (c) 1991 BY                            *
**             DIGITAL EQUIPMENT CORPORATION, MAYNARD, MASS.                *
**                          ALL RIGHTS RESERVED                             *
**                                                                          *
**  THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY BE USED AND  COPIED  *
**  ONLY  IN  ACCORDANCE  WITH  THE  TERMS  OF  SUCH  LICENSE AND WITH THE  *
**  INCLUSION OF THE ABOVE COPYRIGHT NOTICE.  THIS SOFTWARE OR  ANY  OTHER  *
**  COPIES  THEREOF MAY NOT BE PROVIDED OR OTHERWISE MADE AVAILABLE TO ANY  *
**  OTHER PERSON.  NO TITLE TO AND OWNERSHIP OF  THE  SOFTWARE  IS  HEREBY  *
**  TRANSFERRED.                                                            *
**                                                                          *
**  THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE  WITHOUT  NOTICE  *
**  AND  SHOULD  NOT  BE  CONSTRUED  AS  A COMMITMENT BY DIGITAL EQUIPMENT  *
**  CORPORATION.                                                            *
**                                                                          *
**  DIGITAL ASSUMES NO RESPONSIBILITY FOR THE USE OR  RELIABILITY  OF  ITS  *
**  SOFTWARE ON EQUIPMENT WHICH IS NOT SUPPLIED BY DIGITAL.                 *
**                                                                          *
*****************************************************************************
**++
**  PRODUCT:
***
**	DEC C++ Demangler
**
**  FILE:  
**
**      function_type.c
**
**  ABSTRACT:
**
**	Grammar element that parses a function type.
**
**  ENVIRONMENT:
**
**	User Mode
**
**  AUTHORS:
**
** 	Shell S. Simpson (DECwest Engineering)
**
**  MODIFICATION HISTORY:
**
**	20-Mar-1997	Shi-Jung Kao	cxxc_bugs: 4032
**	Add ptr_to_function_type to MLD_function to handle (*(*)())
**
**	27-Oct-1999	Shi-Jung Kao
**	Accept ANSI_FUNCTION_TOKEN.
**
**	02-Feb-2000	Shi-Jung Kao
**	Cleanup, added necessary delete_type_list, delete_function.
**
**	14-Feg-2000	Shi-Jung Kao
**	Added extern_c to MLD_function type.
**	Added DEMANGLE_TRACEIN/TRACEOUT.
**--
**/


#ifdef CXX_NT
#include "defs.h"
#include "mn_nm.h"
#include "gram_elm.h"
#include "util.h"
#else
#include "demangle.h"
#include "grammar_elements.h"
#endif

#include "demangle_internal.h"

#include <stdlib.h>
#include <stddef.h>
#include <stdio.h>
#include <string.h>

/* language element particular definitions */
#define ASCII_ZERO 48

int /* recognition status */
_function_type(const char* s,		/* IN:  string to be parsed */
             int start,			/* IN:  index of first unparsed character */
             int* end,			/* OUT: index of last parsed character */
             struct MLD_function** p)/* OUT: structure containing function (allocated) */
{
  /* variables common to all language elements */
  int ci;  /* current index */
  int cj;
  char ch; /* current character */
  
  /* variables particular to this language element */
  int status;
  int end_of_ge;
  struct MLD_type* fal; /* storage for a function argument list */
  struct MLD_type* rt; /* storage for a return type */
  struct MLD_function* ft = NULL; /* storage for a function type */
  struct MLD_function* temp_p;
  int is_externc = FALSE;

  /* initialization particular to language element */

  DEMANGLE_TRACEIN(_function_type);  

  /*
  **
  ** Check for function Token
  **
  */

  ch = s[ci = start]; /* initialize ch and ci */
  if (ch != FUNCTION_TOKEN && ch != ANSI_FUNCTION_TOKEN)
  {
    *p = 0L;
    *end == start - 1;
    DEMANGLE_TRACEOUT(_function_type);  
    return MLD_NOTHING_RECOGNIZED;
  }
  ch = s[++ci]; /* increment ch and ci */

  /*
   * Check if function has extern "C" linkage.
   */
  if (ch == 'K') {
    ci++;
    is_externc = TRUE;
  }

  /*
  **
  ** Check for function arguments
  **
  */

  status = _function_arguments(s, ci, &end_of_ge, &fal);
  
  switch(status)
  {
    case MLD_NOTHING_RECOGNIZED: /* there must be a type after the function token */
    case MLD_ERROR:
      _delete_type_list(fal); /* deallocate existing function argument list */
      *p = 0L;
      *end = start - 1;
      DEMANGLE_TRACEOUT(_function_type);  
      return MLD_ERROR;
    break;
    
    case MLD_SOMETHING_RECOGNIZED:
      /* normal case, continue */
    break;
  }
  ch = s[ci = (end_of_ge+1)]; /* set ch and ci */
  

  /*
  **
  ** Check for underscore
  **
  */

  if (ch != '_')
  {
    _delete_type_list(fal); /* deallocate existing function argument list */
    *p = 0L;
    *end = start - 1;
    DEMANGLE_TRACEOUT(_function_type);  
    return MLD_NOTHING_RECOGNIZED;
  }
  ch = s[++ci]; /* increment ch and ci */
   

  /*
  **
  ** Check for return type (for now assume that return type is just a type)
  **
  */

  /*
   * This code handles the ambiguous case: cxxc_bugs 4032
   * Source code:
   *		foo(float (*(*f2)(int, float))(int, int),
   *				  int v1, int v2, float w1, float w2)
   * Mangled name:
   *		bar__XPXif_PXii_fiiff
   *
   * Without the following code, demagler shall interpret it as a result of
   * an illegal C++ syntax:
   *	
   *	bar(float (*)(int, int) (*)(int, float), int, int, float, float)
   *
   * Now, we shall interpret it as:
   *
   *	bar(float (*(*)(int, float))(int, int), int, int, float, float)
   *
   */
  if (ch == 'P') {
    /*
     * It has to be the case of (*(*)())
     */
    status = _function_type(s, ci+1, &end_of_ge, &ft);
    if (status == MLD_SOMETHING_RECOGNIZED) {
      ci = end_of_ge;
    }
  }
  status = _type(s, ci, &end_of_ge, &rt);
  
  switch(status)
  {
    case MLD_NOTHING_RECOGNIZED: /* there must be a type after the function token */
    case MLD_ERROR:
      _delete_type_list(fal); /* deallocate existing function argument list */
      _delete_function(ft);
      *p = 0L;
      *end = start - 1;
      DEMANGLE_TRACEOUT(_function_type);  
      return MLD_ERROR;
    break;
    
    case MLD_SOMETHING_RECOGNIZED:
      /* normal case, continue */
    break;
  }
  ch = s[ci = (end_of_ge+1)]; /* set ch and ci */


  /*
  **
  ** Put the function structure together
  **
  */

  temp_p = (struct MLD_function*) malloc(sizeof(struct MLD_function)); 
  DEMANGLE_RECORD_ALLOC(temp_p);
  if (temp_p == 0L) {
    DEMANGLE_TRACEOUT(_function_type);  
    return MLD_ERROR; /* error if no memory allocated */
  }
  temp_p->args = fal; 
  temp_p->return_type = rt;
  temp_p->ptr_to_function_type = ft;
  temp_p->tm_list = 0L;
  temp_p->extern_c = is_externc;

  *p = temp_p;
  *end = end_of_ge;
#if !PRODUCTION 
  if (demangle_dump)
    _dump_function(temp_p);
#endif
  DEMANGLE_TRACEOUT(_function_type);  
  return MLD_SOMETHING_RECOGNIZED;
}
