#pragma required_pointer_size long
#ifndef lint
static const char sccsid[] = "@(#)internal_symbol.c	1.1     2/10/94";
#endif
/*
*****************************************************************************
**                                                                          *
**                         COPYRIGHT (c) 1991 BY                            *
**             DIGITAL EQUIPMENT CORPORATION, MAYNARD, MASS.                *
**                         ALL RIGHTS RESERVED                              *
**                                                                          *
**  THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY BE USED AND  COPIED  *
**  ONLY  IN  ACCORDANCE  WITH  THE  TERMS  OF  SUCH  LICENSE AND WITH THE  *
**  INCLUSION OF THE ABOVE COPYRIGHT NOTICE.  THIS SOFTWARE OR  ANY  OTHER  *
**  COPIES  THEREOF MAY NOT BE PROVIDED OR OTHERWISE MADE AVAILABLE TO ANY  *
**  OTHER PERSON.  NO TITLE TO AND OWNERSHIP OF  THE  SOFTWARE  IS  HEREBY  *
**  TRANSFERRED.                                                            *
**                                                                          *
**  THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE  WITHOUT  NOTICE  *
**  AND  SHOULD  NOT  BE  CONSTRUED  AS  A COMMITMENT BY DIGITAL EQUIPMENT  *
**  CORPORATION.                                                            *
**                                                                          *
**  DIGITAL ASSUMES NO RESPONSIBILITY FOR THE USE OR  RELIABILITY  OF  ITS  *
**  SOFTWARE ON EQUIPMENT WHICH IS NOT SUPPLIED BY DIGITAL.                 *
**                                                                          *
*****************************************************************************
**++
**  PRODUCT:
***
**	DEC C++ Demangler
**
**  FILE:  
**
**      internal_symbol.c
**
**  ABSTRACT:
**
**	Grammar element that parses a basic_type.
**
**  ENVIRONMENT:
**
**	User Mode
**
**  AUTHORS:
**
** 	Shell S. Simpson (DECwest Engineering)
**
**  MODIFICATION HISTORY:
**
**      $Id: 
**      $Log: 
**
**--
**/


#ifdef CXX_NT
#include "int_sym.h"
#else
#include "demangle.h"
#endif

#include <stddef.h>
#include <stdlib.h>

#include <string.h>

int /* recognition status */
MLD_internal_symbol(const char* os,		/* IN:  string to be parsed */
           		int start,		/* IN:  index of first unparsed character */
           		int *symbol)	/* OUT:	enumeration value of symbol */
{
  /* variables common to all language elements */
  int ci;  /* current index */
  char ch; /* current character */
  
  int sym; /* temporary index variable used for symbol number*/
  char* in;
  int ip, arg;
  int length_of_in;
  int junk; /* this variable should be unnecessary */
  int length_of_s;
  char *s;

  /* allocate storage for s */
  length_of_s = strlen(os);
  s = (char *) malloc(length_of_s+1);

  if (s == 0L) return MLD_ERROR;

  /* normalize string (i.e. remove trailing cr) */  
  strcpy(s, os);
  if (s[length_of_s-1] == '\n') s[length_of_s-1] = 0;

  length_of_s = strlen(s);

  /*
  ***
  *** Check to see if the matches the current internal symbol.
  *** If so, return the corresponding internal symbol number 
  */

  for (sym = 0 ; sym < MLD_NUMBER_OF_INTERNAL_SYMBOLS ; sym++)
  {
    /* extract information about the symbol we're comparing */
    in = MLD_internalNameTable[sym].s;
    length_of_in = strlen(in);
    ip = MLD_internalNameTable[sym].ip;
    arg = MLD_internalNameTable[sym].arg;
    
    if ((ip == MLD_all) && (length_of_s != length_of_in)) continue;
        
    switch (ip)
    {
      case MLD_all:
        if (strncmp(s, in, length_of_in) == 0)
        {
          *symbol = sym;
	  free(s);
          return MLD_SOMETHING_RECOGNIZED;
        }
      break;
 
      case MLD_pre:
        if (strncmp(s, in, length_of_in) == 0)
        {
          *symbol = sym;
	  free(s);
          return MLD_SOMETHING_RECOGNIZED;
        }
      break;

      case MLD_post:
        if ((length_of_in <= length_of_s) && 
            (strncmp(&s[length_of_s-length_of_in], in, length_of_in) == 0))
        {
          *symbol = sym;
	  free(s);
          return MLD_SOMETHING_RECOGNIZED;
        }
      break;
    }
  }
  free(s);
  return MLD_NOTHING_RECOGNIZED;  
}
