#pragma required_pointer_size long
#ifndef lint
static const char sccsid[] = "@(#)mangled_template_name.c	1.1     2/10/94";
#endif
/*
*****************************************************************************
**                                                                          *
**                         COPYRIGHT (c) 1991, 1996 BY                      *
**             DIGITAL EQUIPMENT CORPORATION, MAYNARD, MASS.                *
**                          ALL RIGHTS RESERVED                             *
**                                                                          *
**  THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY BE USED AND  COPIED  *
**  ONLY  IN  ACCORDANCE  WITH  THE  TERMS  OF  SUCH  LICENSE AND WITH THE  *
**  INCLUSION OF THE ABOVE COPYRIGHT NOTICE.  THIS SOFTWARE OR  ANY  OTHER  *
**  COPIES  THEREOF MAY NOT BE PROVIDED OR OTHERWISE MADE AVAILABLE TO ANY  *
**  OTHER PERSON.  NO TITLE TO AND OWNERSHIP OF  THE  SOFTWARE  IS  HEREBY  *
**  TRANSFERRED.                                                            *
**                                                                          *
**  THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE  WITHOUT  NOTICE  *
**  AND  SHOULD  NOT  BE  CONSTRUED  AS  A COMMITMENT BY DIGITAL EQUIPMENT  *
**  CORPORATION.                                                            *
**                                                                          *
**  DIGITAL ASSUMES NO RESPONSIBILITY FOR THE USE OR  RELIABILITY  OF  ITS  *
**  SOFTWARE ON EQUIPMENT WHICH IS NOT SUPPLIED BY DIGITAL.                 *
**                                                                          *
*****************************************************************************
**++
**  PRODUCT:
***
**	DEC C++ Demangler
**
**  FILE:  
**
**      mangled_template_name.c
**
**  ABSTRACT:
**
**	Grammar element that parses a mangled_name.
**
**  ENVIRONMENT:
**
**	User Mode
**
**  AUTHORS:
**
** 	Shell S. Simpson (DECwest Engineering)
**
**  MODIFICATION HISTORY:
**
**       9-July-1996	Harold Seigel
**      Fix strcpy memory corruption.
**
**      13-Aug-1996	Harold Seigel
**      Fix heap leak by deallocating a_type when an error is found.
**
**      29-Oct-1997	Shi-Jung Kao
**      Skip the special attribute __S if the function is specialized.
**
**	03-Apr-1998	Shi-Jung Kao
**	_delete_type_list(a) to free all mallocs.
**
**	22-Oct-1999	Shi-Jung Kao
**	Skip the special attribute __S if the template is specialized.
**	Now, we have the fact the template from which this function 
**	is generated is specialized.  Also support the ill formed
**	partial specilization. i.e. __TP__Ti.
**
**      01-Mar-2000	Shi-Jung Kao
**	Added DEMANGLE_TRACEIN/TRACEOUT.
**--
**/


#ifdef CXX_NT
#include "defs.h"
#include "mn_nm.h"
#include "gram_elm.h"
#include "util.h"
#else
#include "demangle.h"
#include "grammar_elements.h"
#endif

#include "demangle_internal.h"

#include <stdlib.h>
#include <stddef.h>
#include <stdio.h>
#include <string.h>

/* language element particular definitions */
#define ASCII_ZERO 48

int /* recognition status */
_mangled_template_name
  (const char* s,		/* IN:  string to be parsed */
     int start,			/* IN:  index of first unparsed character */
     int* end,			/* OUT: index of last parsed character */
     struct MLD_mangled_template_name** mtn,
				/* OUT: struct containing mangled templ name */
     char* str_before_name,	/* OUT: string before mangled templ name */
     char* str_replaced_by_name,
				/* OUT: string replaced by mangled templ name*/
     char* str_after_name,	/* OUT: string after mangled templ name */
     int size_of_string)	/* IN:	max size of the strings listed above */
{
  /* variables common to all language elements */
  int ci;  /* current index */
  char ch; /* current character */
  
  /* variables particular to this language element */
  int status;
  int end_of_grammar_component;
  struct MLD_mangled_template_name* temp_mn;
  char bs[STRSIZ];		/* string before demangled string */
  char rs[STRSIZ];		/* string replaced by demangled string */
  char as[STRSIZ];		/* string after demangled string */
  char id[STRSIZ];		/* identifier string */
  char q[STRSIZ]; 		/* qualifier string */
  char class[STRSIZ];		/* class string */
  struct MLD_type* a_type=0;	/* function argument */
  int end_of_mn;		/* index of last character in s of mn */

  int num_du = 0;		/* number of double underscores */
  int du_count;                 /* looping variable that determines the
                                   underscore number that will be used
                                   in the identifier */
  
  DEMANGLE_TRACEIN(_mangled_template_name);

  num_du = _num_dbl_uscores(s, start);

  for (du_count=1; du_count<=num_du; du_count++)
  { /* start of loop that repeatled check for double underscores */  


  /* initialization particular to language element */
  temp_mn = 0L;
  *mtn = 0L;
  
  /*
  **
  ** Get the identifier
  **
  */

  ch = s[ci = start]; /* initialize ci and ch */
  status = _identifier(s, ci, &end_of_grammar_component, du_count, 
                                           id, STRSIZ, bs, STRSIZ);
  switch (status)
  {    
    case MLD_NOTHING_RECOGNIZED:
      continue;
    break;
    
    case MLD_ERROR:
      /* if error occured, return ERROR */
      *end = start - 1; /* nothing recognized */
      *mtn = 0L; /* no mangled name */
      
      if (size_of_string > 0) /* copy if enough room */
      {
        strcpy(str_before_name,"");
        strcpy(str_replaced_by_name,"");
      }
      
      if ((strlen(s) - start) < size_of_string) /* copy if enough room */
      {
        strcpy(str_after_name, &s[start]); /* everything is "after" */
      }
      
      DEMANGLE_TRACEOUT(_mangled_template_name);
      return MLD_ERROR;
    break;
  }
  ch = s[ci = (end_of_grammar_component+1)]; /* update ci and ch */


  /*
  **
  ** Get the "__S" if the template from which this function is generated 
  ** is specialized.
  **
  */

  if (((s[ci] == '_') && (s[ci+1] == '_') && (s[ci+2] == SPECIAL_TOKEN))) {
    end_of_grammar_component = end_of_grammar_component + 3;
    ch = s[ci = (ci+3)]; /* update ci and ch */
  }

  /*
  **
  ** Get the "__T" or ANSI form "__tm__"
  ** Note, we are supporting arm mode of __TP__Ti of partial specialization.
  ** This is bad because the ambiguity of it.  --SJK 22 Oct,1999
  ** Besides, it should be __TPi instead of __TP__Ti.
  **
  */

  if (strncmp(&s[ci], "__T", 3) == 0) {
    int saved_ci;
    ch = s[ci = (ci+3)]; /* update ci and ch */
    saved_ci = ci;
    while (ch == 'P') {
      ch = s[++ci];
    }
    if (strncmp(&s[ci], "__T", 3) == 0) {
      ch = s[ci = (ci+3)]; /* update ci and ch */
    } else {
      ci = saved_ci;
      ch = s[ci];
    }
  }
#if DEC_ANSI_FORM
  else if (strncmp(&s[ci], "__tm__", 6) == 0) {
    ch = s[ci = (ci+6)]; /* update ci and ch */
    du_count++; /* also update the for loop du_count */
  }
#endif
  else
    continue;

  /* get rid of previous function arguments, if any */
  _delete_type_list(a_type);

  /*
  **
  ** Get the arguments
  **
  */

  status = _template_arguments(s, ci, &end_of_grammar_component, &a_type);

  switch (status)
  {    
    case MLD_NOTHING_RECOGNIZED:
      continue;
    break;
    
    case MLD_ERROR:
      /* if error occured, return ERROR */
      *end = start - 1; /* nothing recognized */
      *mtn = 0L; /* no mangled name */
      
      if (size_of_string > 0) /* copy if enough room */
      {
        strcpy(str_before_name,"");
        strcpy(str_replaced_by_name,"");
      }
      
      if ((strlen(s) - start) < size_of_string) /* copy if enough room */
      {
        strcpy(str_after_name, &s[start]); /* everything is "after" */
      }
      
      DEMANGLE_TRACEOUT(_mangled_template_name);
      return MLD_ERROR;
    break;    
  }
  ch = s[ci = (end_of_grammar_component+1)]; /* update ci and ch */

  /*
  **
  ** Get the "__S" if this function is specialized.
  **
  */

  if (((s[ci] == '_') && (s[ci+1] == '_') && (s[ci+2] == SPECIAL_TOKEN))) {
    end_of_grammar_component = end_of_grammar_component + 3;
    ch = s[ci = (ci+3)]; /* update ci and ch */
  }

  /*
  **
  ** Check if there is an external suffix, if there is discard it
  **
  */

  status = _external_suffix(s, end_of_grammar_component+1, 
					&end_of_grammar_component);

  switch (status)
  {
    case MLD_NOTHING_RECOGNIZED: /* OK not to recognize */
    break;

    case MLD_ERROR:
      /* get rid of function arguments, if any */
      _delete_type_list(a_type);

      /* if error occured, return ERROR */
      *end = start - 1; /* nothing recognized */
      *mtn = 0L; /* no mangled name */

      if (size_of_string > 0) /* copy if enough room */
      {
        strcpy(str_before_name,"");
        strcpy(str_replaced_by_name,"");
      }

      if ((strlen(s) - start) < size_of_string) /* copy if enough room */
      {
        strcpy(str_after_name, &s[start]); /* everything is "after" */
      }

      DEMANGLE_TRACEOUT(_mangled_template_name);
      return MLD_ERROR;
    break;
  }
  ch = s[ci = (end_of_grammar_component+1)]; /* update ci and ch */
 

  /*
  **
  ** Check for white space
  **
  */
  
  end_of_mn = end_of_grammar_component;
  if (end_of_mn < strlen(s)) {

    status = _white_space(s, end_of_grammar_component + 1, 
					&end_of_grammar_component);

    switch (status) {    
    case MLD_NOTHING_RECOGNIZED:
      continue;
    break;
    
    case MLD_ERROR:
      /* if error occured, return ERROR */
      
      /* get rid of function arguments, if any */
      _delete_type_list(a_type);
      
      *end = start - 1; /* nothing recognized */
      *mtn = 0L; /* no mangled name */
      
      if (size_of_string > 0) /* copy if enough room */
      {
        strcpy(str_before_name,"");
        strcpy(str_replaced_by_name,"");
      }
      
      if ((strlen(s) - start) < size_of_string) /* copy if enough room */
      {
        strcpy(str_after_name, &s[start]); /* everything is "after" */
      }
      
      DEMANGLE_TRACEOUT(_mangled_template_name);
      return MLD_ERROR;
      break;
    }
  }
 
  /*
  **
  ** All the trials have been passed.  Now is the type to create the
  ** mangled name structure and set it appropriately.
  **
  */

 
  temp_mn = (struct MLD_mangled_template_name*) 
		malloc(sizeof(struct MLD_mangled_template_name));
  DEMANGLE_RECORD_ALLOC(temp_mn);
  if (temp_mn == 0L) {
    DEMANGLE_TRACEOUT(_mangled_template_name);
    return MLD_ERROR; /* error if no memory allocated */
  }

  temp_mn->identifier = (char*) malloc(strlen(id)+1);
  DEMANGLE_RECORD_ALLOC(temp_mn->identifier);
  if (temp_mn->identifier == 0L)
  {
     /* get rid of function arguments, if any */
     _delete_type_list(a_type);

    DEMANGLE_FREE(temp_mn);
    DEMANGLE_TRACEOUT(_mangled_template_name);
    return MLD_ERROR; /* error if no memory allocated */
  }
  
  strcpy(temp_mn->identifier, id);
  
  temp_mn->args = a_type;
      
  if (strlen(bs) < size_of_string) /* copy if enough room */
  {
    strcpy(str_before_name, bs);
  }

  /* copy if enough room */
  if ((end_of_mn - (strlen(bs) + start) + 1) < size_of_string)
  {
    strncpy(str_replaced_by_name,
            &s[start + strlen(bs)],
            end_of_mn - (strlen(bs) + start) + 1 );
    /* null terminate */
    str_replaced_by_name[(end_of_mn - (strlen(bs) + start) + 1)] = 0;
  }
      
  if (strlen(&s[end_of_mn+1]) < size_of_string) /* copy if enough room */
  {
    strcpy(str_after_name, &s[end_of_mn+1]);
  }
 
  *mtn = temp_mn;   
  *end = end_of_mn; /* end of mangled name */
  DEMANGLE_TRACEOUT(_mangled_template_name);
  return MLD_SOMETHING_RECOGNIZED;
  
  } /* end of loop that repeatedly checks for double underscores */
  
  
  /* if no identifier was recognized, return NOTHING_RECOGNIZED */
  *end = start - 1; /* nothing recognized */
  *mtn = 0L; /* no mangled name */
 
  /* get rid of function arguments, if any */
  _delete_type_list(a_type);

  if ((strlen(s) - start) < size_of_string) /* copy if enough room */
  {
    strcpy(str_before_name, &s[start]); 
  }
      
  if (size_of_string > 0) /* copy if enough room */
  {
    strcpy(str_replaced_by_name,"");
    strcpy(str_after_name,"");
  }
      
  DEMANGLE_TRACEOUT(_mangled_template_name);
  return MLD_NOTHING_RECOGNIZED;
}
