#pragma required_pointer_size long
#ifndef lint
static const char sccsid[] = "@(#)pointer_to_member_type.c	1.1     2/10/94";
#endif
/*
*****************************************************************************
**                                                                          *
**                         COPYRIGHT (c) 1991 BY                            *
**             DIGITAL EQUIPMENT CORPORATION, MAYNARD, MASS.                *
**                          ALL RIGHTS RESERVED                             *
**                                                                          *
**  THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY BE USED AND  COPIED  *
**  ONLY  IN  ACCORDANCE  WITH  THE  TERMS  OF  SUCH  LICENSE AND WITH THE  *
**  INCLUSION OF THE ABOVE COPYRIGHT NOTICE.  THIS SOFTWARE OR  ANY  OTHER  *
**  COPIES  THEREOF MAY NOT BE PROVIDED OR OTHERWISE MADE AVAILABLE TO ANY  *
**  OTHER PERSON.  NO TITLE TO AND OWNERSHIP OF  THE  SOFTWARE  IS  HEREBY  *
**  TRANSFERRED.                                                            *
**                                                                          *
**  THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE  WITHOUT  NOTICE  *
**  AND  SHOULD  NOT  BE  CONSTRUED  AS  A COMMITMENT BY DIGITAL EQUIPMENT  *
**  CORPORATION.                                                            *
**                                                                          *
**  DIGITAL ASSUMES NO RESPONSIBILITY FOR THE USE OR  RELIABILITY  OF  ITS  *
**  SOFTWARE ON EQUIPMENT WHICH IS NOT SUPPLIED BY DIGITAL.                 *
**                                                                          *
*****************************************************************************
**++
**  PRODUCT:
***
**	DEC C++ Demangler
**
**  FILE:  
**
**      pointer_to_member_type.c
**
**  ABSTRACT:
**
**	Grammar element that parses a pointer to member type.
**
**  ENVIRONMENT:
**
**	User Mode
**
**  AUTHORS:
**
** 	Shell S. Simpson (DECwest Engineering)
**
**  MODIFICATION HISTORY:
**
**  19-Feb-1998	 Shi-Jung Kao
**	_delete_qualifications before return ERROR/NOTHING_RECOGNIZED.
**
**  01-Mar-2000  Shi-Jung Kao
**	Added DEMANGLE_TRACEIN/TRACEOUT.
**
**--
**/


#ifdef CXX_NT
#include "defs.h"
#include "mn_nm.h"
#include "gram_elm.h"
#include "util.h"
#else
#include "demangle.h"
#include "grammar_elements.h"
#endif

#include "demangle_internal.h"

#include <stdlib.h>
#include <stddef.h>
#include <stdio.h>
#include <string.h>

/* language element particular definitions */
#define ASCII_ZERO 48

int /* recognition status */
_pointer_to_member_type(const char* s,	/* IN:  string to be parsed */
	int start,		/* IN:  index of first unparsed character */
	int* end,		/* OUT: index of last parsed character */
	struct MLD_pointer_to_member** p)
		/* OUT: structure containing pointer_to_member (allocated) */
{
  /* variables common to all language elements */
  int ci;  /* current index */
  char ch; /* current character */
  
  /* variables particular to this language element */
  int status;
  int end_of_gc;
  struct MLD_type* t; /* storage for a type */
  struct MLD_pointer_to_member* temp_p;
  struct MLD_qualification* q;

  /* initialization particular to language element */

  DEMANGLE_TRACEIN(_pointer_to_member_type);

  /*
  **
  ** Check for Pointer_to_member Token
  **
  */

  ch = s[ci = start]; /* initialize ch and ci */
  if (ch != POINTER_TO_MEMBER_TOKEN)
  {
    *p = 0L;
    *end == start - 1;
    DEMANGLE_TRACEOUT(_pointer_to_member_type);
    return MLD_NOTHING_RECOGNIZED;
  }
  ch = s[++ci]; /* increment ch and ci */


  /*
  **
  ** Check for qualifications
  **
  */

  status = _parse_qualifications(s, ci, &end_of_gc, &q);
  
  switch(status)
  {
    case MLD_NOTHING_RECOGNIZED: /* there must be a qualification after
                                the Pointer to member token */
    case MLD_ERROR:
      *p = 0L;
      *end = start - 1;
      DEMANGLE_TRACEOUT(_pointer_to_member_type);
      return MLD_ERROR;
      break;
    
    case MLD_SOMETHING_RECOGNIZED:
      /* normal case, continue */
      break;

    default:
      break;
  }
  ch = s[ci = (end_of_gc + 1)]; /* the next character is one after the end of type */

  /*
  **
  ** Check for type
  **
  */

  status = _type(s, ci, &end_of_gc, &t);
  
  switch(status)
  {
    case MLD_NOTHING_RECOGNIZED: /* there must be a type after the Pointer to member token */
    case MLD_ERROR:
      *p = 0L;
      *end = start - 1;
      _delete_qualifications(q);
      DEMANGLE_TRACEOUT(_pointer_to_member_type);
      return MLD_ERROR;
    break;
    
    case MLD_SOMETHING_RECOGNIZED:
      /* normal case, continue */
    break;
  }

  /*
  **
  ** Put the pointer_to_member structure together
  **
  */

  temp_p = (struct MLD_pointer_to_member*) malloc(sizeof(struct MLD_pointer_to_member));
  DEMANGLE_RECORD_ALLOC(temp_p);
  if (temp_p == 0L) {
    DEMANGLE_TRACEOUT(_pointer_to_member_type);
    return MLD_ERROR; /* error if no memory allocated */
  }

  temp_p->qualifications = q;
  temp_p->type_modified = t;
  
  *p = temp_p;
  *end = end_of_gc; 
  DEMANGLE_TRACEOUT(_pointer_to_member_type);
  return MLD_SOMETHING_RECOGNIZED;
}
