#pragma required_pointer_size long
#ifndef lint
static const char sccsid[] = "@(#)type_modifiers.c	1.1     2/10/94";
#endif
/*
*****************************************************************************
**                                                                          *
**                         COPYRIGHT (c) 1991 BY                            *
**             DIGITAL EQUIPMENT CORPORATION, MAYNARD, MASS.                *
**                          ALL RIGHTS RESERVED                             *
**                                                                          *
**  THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY BE USED AND  COPIED  *
**  ONLY  IN  ACCORDANCE  WITH  THE  TERMS  OF  SUCH  LICENSE AND WITH THE  *
**  INCLUSION OF THE ABOVE COPYRIGHT NOTICE.  THIS SOFTWARE OR  ANY  OTHER  *
**  COPIES  THEREOF MAY NOT BE PROVIDED OR OTHERWISE MADE AVAILABLE TO ANY  *
**  OTHER PERSON.  NO TITLE TO AND OWNERSHIP OF  THE  SOFTWARE  IS  HEREBY  *
**  TRANSFERRED.                                                            *
**                                                                          *
**  THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE  WITHOUT  NOTICE  *
**  AND  SHOULD  NOT  BE  CONSTRUED  AS  A COMMITMENT BY DIGITAL EQUIPMENT  *
**  CORPORATION.                                                            *
**                                                                          *
**  DIGITAL ASSUMES NO RESPONSIBILITY FOR THE USE OR  RELIABILITY  OF  ITS  *
**  SOFTWARE ON EQUIPMENT WHICH IS NOT SUPPLIED BY DIGITAL.                 *
**                                                                          *
*****************************************************************************
**++
**  PRODUCT:
***
**	DEC C++ Demangler
**
**  FILE:  
**
**      type_modifiers.c
**
**  ABSTRACT:
**
**	Grammar element that parses type modifiers.
**
**  ENVIRONMENT:
**
**	User Mode
**
**  AUTHORS:
**
** 	Shell S. Simpson (DECwest Engineering)
**
**  MODIFICATION HISTORY:
**
**      06-Feb-1995	Harold Seigel
**      Add support for __unaligned.
**
**--
**/


#ifdef CXX_NT
#include "defs.h"
#include "mn_nm.h"
#include "gram_elm.h"
#include "util.h"
#else
#include "demangle.h"
#include "grammar_elements.h"
#endif

#include "demangle_internal.h"

#include <stdlib.h>
#include <stddef.h>
#include <stdio.h>
#include <string.h>

/* language element particular definitions */
#define ASCII_ZERO 48

int /* recognition status */
_type_modifiers
	(const char* s,					/* IN:  string to be parsed */
     int start,						/* IN:  index of first unparsed character */
     int* end,						/* OUT: index of last parsed character */
     struct MLD_type_modifiers** tml)	/* OUT: type modifier list (allocates memory) */
{
  /* variables common to all language elements */
  int ci;  /* current index */
  char ch; /* current character */
  struct MLD_type_modifiers* temp_tml; /* temporary type modifier list */ 

  int ct = 0; /* type modifiers */
  int st = 0;
  int ut = 0; 
  int vt = 0; 
  int unt = 0; 
  
  /* initial values */
  *tml = 0L;
  *end = start - 1;
  ch = s[ci = start];
  
  /*
  **
  ** Check for const token (checking in alphabetical order)
  **
  */
  
  if (ch == CONST_TOKEN)
  {
    ct = 1;
    ch = s[++ci];
  };
  
  
  /*
  **
  ** Check for signed token
  **
  */
  
  if (ch == SIGNED_TOKEN)
  {
    st = 1;
    ch = s[++ci];
  };
  
  
  /*
  **
  ** Check for unsigned token
  **
  */
  
  if (ch == UNSIGNED_TOKEN)
  {
    ut = 1;
    ch = s[++ci];
  };
  
  /*
  **
  ** Check for volatile token
  **
  */
  
  if (ch == VOLATILE_TOKEN)
  {
    vt = 1;
    ch = s[++ci];
  };
  
  /*
  **
  ** Check for __unaligned token
  **
  */
  
  if (ch == UNALIGNED_TOKEN)
  {
    unt = 1;
    ch = s[++ci];
  };
  
  /*
  **
  ** Put it all together
  **
  */
  
  if (ct || st || ut || vt || unt)
  {
    temp_tml = (struct MLD_type_modifiers*) malloc(sizeof(struct MLD_type_modifiers)); 
    DEMANGLE_RECORD_ALLOC(temp_tml);
    if (temp_tml == 0L) return MLD_ERROR; /* error, no memory allocated */
  
    temp_tml->const_tm = ct;
    temp_tml->signed_tm = st;
    temp_tml->unsigned_tm = ut;
    temp_tml->volatile_tm = vt;
    temp_tml->unaligned_tm = unt;
  
    *tml = temp_tml;
    *end = ci - 1; 
    return MLD_SOMETHING_RECOGNIZED;
  }
  else
  {
    *tml = 0L;
    return MLD_NOTHING_RECOGNIZED;
  }
}
