#ifndef lint
/* static const char sccsid[] = "@(#)utilities.h	1.1     2/10/94"; */
#endif
/*
*****************************************************************************
**                                                                          *
**                         COPYRIGHT (c) 1991 BY                            *
**             DIGITAL EQUIPMENT CORPORATION, MAYNARD, MASS.                *
**                          ALL RIGHTS RESERVED                             *
**                                                                          *
**  THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY BE USED AND  COPIED  *
**  ONLY  IN  ACCORDANCE  WITH  THE  TERMS  OF  SUCH  LICENSE AND WITH THE  *
**  INCLUSION OF THE ABOVE COPYRIGHT NOTICE.  THIS SOFTWARE OR  ANY  OTHER  *
**  COPIES  THEREOF MAY NOT BE PROVIDED OR OTHERWISE MADE AVAILABLE TO ANY  *
**  OTHER PERSON.  NO TITLE TO AND OWNERSHIP OF  THE  SOFTWARE  IS  HEREBY  *
**  TRANSFERRED.                                                            *
**                                                                          *
**  THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE  WITHOUT  NOTICE  *
**  AND  SHOULD  NOT  BE  CONSTRUED  AS  A COMMITMENT BY DIGITAL EQUIPMENT  *
**  CORPORATION.                                                            *
**                                                                          *
**  DIGITAL ASSUMES NO RESPONSIBILITY FOR THE USE OR  RELIABILITY  OF  ITS  *
**  SOFTWARE ON EQUIPMENT WHICH IS NOT SUPPLIED BY DIGITAL.                 *
**                                                                          *
*****************************************************************************
**++
**  PRODUCT:
***
**	DEC C++ Demangler
**
**  FILE:  
**
**      utilities.h
**
**  ABSTRACT:
**
**	This module contains functions that deallocate storage.
**
**  ENVIRONMENT:
**
**	User Mode
**
**  AUTHORS:
**
** 	Shell S. Simpson (DECwest Engineering)
**
**  MODIFICATION HISTORY:
**
**  15-Jun-1998	Shi-Jung Kao
**	Added all dump routines.
**
**  28-Oct-1998 Shi-Jung Kao
**	Added _concat_complex_notemplatearg.
**
**  17-Jan-2000 Shi-Jung Kao
**	Added no_template_details. i.e. no <T0=...>
**	Added _init_shared_id().
**	Added _alloc_nontype().
**
**--
**/


#ifndef _UTILITIES_H_
#define _UTILITIES_H_

#ifdef CXX_NT
#include "mn_nm.h"
#else
#include "mangled_name.h"
#endif

/* 
***
*** General Utility Functions
***
*/

void /* substitute string or null, if nothing found */
_basic_operator_substitute(const char* ts,	/* IN:  operator token */
                           char* s,			/* OUT:	operator string */
                           int size_of_s);	/* IN:	max size of s */

int /* recognition status */
_get_class(struct MLD_qualification* q,	/* qualification */
          struct MLD_complex** c); 		/* class */

int /* number of double underscores found */
_num_dbl_uscores(const char* s,	/* IN:  string to be parsed */
          		int start);	/* IN:  index of first unparsed character */

char* /* prepend d to s, put results in s, and return s. */
demangle_string_prepend (char* s, 
			 int max_s, 
			 const char* d);

int /* recognition status */
demangle_unnamed(const char *s, /* IN: string to be parsed */
		 char *string);	/* OUT: of unnamed... */
/* 
***
*** Deletion Utility Functions
***
*/

void
_delete_type_modifier_list(struct MLD_type_modifiers* tml);

struct MLD_type* /* returns the "next" structure */
_delete_type_structure(struct MLD_type* t);

void 
_delete_unmodified_type(struct MLD_unmodified_type* ut);

void
_delete_type_list(struct MLD_type* tl);

void
_delete_mangled_function_name(struct MLD_mangled_function_name* mfn);

void
_delete_mangled_template_name(struct MLD_mangled_template_name* mtn);

void
_delete_mangled_variable_name(struct MLD_mangled_variable_name* mvn);

void
_delete_complex(struct MLD_complex* c);

void
_delete_pointer(struct MLD_pointer* p);

void
_delete_qualifications(struct MLD_qualification* ql);

void
_delete_reference(struct MLD_reference* r);

void
_delete_array(struct MLD_array* a);

void
_delete_pointer_to_member(struct MLD_pointer_to_member* pm);

void
_delete_function(struct MLD_function* f);

#if DEC_ANSI_FORM
void
_delete_address(struct MLD_address* ad);

void
_delete_nontype(struct MLD_nontype* nt);

struct MLD_nontype*
_alloc_nontype();

void
_init_shared_id();

void 
_set_shared_id(struct MLD_unmodified_type *ut);
#endif
/* 
***
*** Concatenation Utility Functions
***
*/

char*
_concat(char* s,
 	   int max_s,
  	   const char* d);

void
_concat_type_modifier_list(struct MLD_type_modifiers* tml, 
                          char* s,
                          int max_s, 
                          int flags );
                          
void
_concat_unmodified_type(struct MLD_unmodified_type* ut,
                      char* s,
                      int max_s, 
                      int flags);
                      
struct MLD_type* /* returns the "next" structure */
_concat_type_structure(struct MLD_type* t,
                      char* s,
                      int max_s, 
                      int flags);
                      
void
_concat_type_list(struct MLD_type* tl,
                 char* s,
                 int max_s, 
                 int flags);
                 
void
_concat_mangled_function_name(struct MLD_mangled_function_name* mfn,
                    char* s,
                    int max_s, 
                    int flags);

void
_concat_mangled_variable_name(struct MLD_mangled_variable_name* mvn,
                    char* s,
                    int max_s, 
                    int flags);                    
                    
void
_concat_complex(struct MLD_complex* c,
               char* s,
               int max_s, 
               int flags);

void
_concat_complex_notemplatearg(struct MLD_complex* c,
               char* s,
               int max_s, 
               int flags);

void
_concat_basic(int bt,
               char* s,
               int max_s, 
               int flags);

void
_concat_pointer(struct MLD_pointer* p,
               char* s,
               int max_s, 
               int flags);
               
void
_concat_qualifications(struct MLD_qualification* ql, 
                      char* s,
                      int max_s, 
                      int flags );

void
_concat_reference(struct MLD_reference* r,
                 char* s,
                 int max_s, 
                 int flags);
                 
void
_concat_array(struct MLD_array* a,
             char* s,
             int max_s, 
             int flags);
             
void
_concat_pointer_to_member(struct MLD_pointer_to_member* pm,
                         char* s,
                         int max_s, 
                         int flags);
                         
void
_concat_function(struct MLD_function* f,
                char* s,
                int max_s, 
                int flags);

#if DEC_ANSI_FORM
void
_concat_address(struct MLD_address* ad,
               char* s,
               int max_s, 
               int flags);
#endif
/* 
***
*** Dump Utility Functions
***
*/

void
_dump_type_modifier_list(struct MLD_type_modifiers* tml);

struct MLD_type* /* returns the "next" structure */
_dump_type_structure(struct MLD_type* t);

void 
_dump_unmodified_type(struct MLD_unmodified_type* ut);

void
_dump_type_list(struct MLD_type* tl);

void
_dump_mangled_function_name(struct MLD_mangled_function_name* mfn);

void
_dump_mangled_template_name(struct MLD_mangled_template_name* mtn);

void
_dump_mangled_variable_name(struct MLD_mangled_variable_name* mvn);

void
_dump_complex(struct MLD_complex* c);

void
_dump_pointer(struct MLD_pointer* p);

void
_dump_qualifications(struct MLD_qualification* ql);

void
_dump_reference(struct MLD_reference* r);

void
_dump_array(struct MLD_array* a);

void
_dump_pointer_to_member(struct MLD_pointer_to_member* pm);

void
_dump_function(struct MLD_function* f);

#if DEC_ANSI_FORM
void
_dump_address(struct MLD_address* ad);
#endif
#endif /* _UTILITIES_H_ */
/* DON'T ADD ANYTHING AFTER THIS #endif */
